#include "defs.h"
#include "core.h"

#include "d1q3.h"
#include "d2q5.h"
#include "d2q9.h"
#include "d3q7.h"
#include "d3q27.h"

template < typename TYPE >
void execute(
	int pde_type=NSE,		// Navier-Stokes (NSE) or Advection Diffusion (ADE) eqs
	int order=4,			// Taylor order
	int max_factors_common_order=4,	// maximal common order for factors
	bool use_symbolic_coefs=false	// use symbolic substitutions
)
{
	CORE<TYPE> core;
	core.ORDER = order;
	core.MFCO = max_factors_common_order;
	core.lbm.pde_type = pde_type;
	core.use_symbolic_coefs=use_symbolic_coefs;

	sprintf(core.target_dir,"output_%s_%s_%s_%s_%d_%d%s",(pde_type==NSE)?"nse":"ade",core.lbm.id, core.lbm.id_col,TYPE::T_MM::id,core.ORDER,core.MFCO,(core.use_symbolic_coefs)?"_symbolic":"");
	mkdir(core.target_dir,0777);

	core.log("LBMAN: d=%d q=%d order=%d mfco=%d model=%s_%s M=%s pde_type=%s do_subs=%s pid=%d", core.D, core.Q, core.ORDER, core.MFCO, core.lbm.id, core.lbm.id_col, TYPE::T_MM::id, pde_type==NSE ? "NSE" : "ADE", (core.use_symbolic_coefs)?"yes":"no",getpid());
	core.verbosity=1;

	core.compute_all();

	core.log("Finished");
}


template< typename MM=D1Q3_M1 >
void d1q3(int pde_type, int order=4, int max_factors_common_order=4, bool use_symbolic=false)
{
	execute<D1Q3_SRT<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D1Q3_MRT1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D1Q3_CLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
}

template< typename MM=D2Q5_M1 >
void d2q5(int pde_type, int order=4, int max_factors_common_order=4, bool use_symbolic=false)
{
	execute<D2Q5_SRT<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q5_MRT1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q5_MRT2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q5_CLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q5_CLBM2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
}


template< typename MM=D2Q9_M1 >
void d2q9(int pde_type, int order=4, int max_factors_common_order=4, bool use_symbolic=false)
{
	execute<D2Q9_SRT<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q9_MRT1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q9_MRT2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q9_CLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q9_CLBM2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q9_CuLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D2Q9_CuLBM2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
}


template< typename MM=D3Q7_M1 >
void d3q7(int pde_type, int order=4, int max_factors_common_order=4, bool use_symbolic=false)
{
	execute<D3Q7_SRT<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q7_MRT1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q7_MRT2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q7_CLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q7_CLBM2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
}


template< typename MM=D3Q27_M1 >
void d3q27(int pde_type, int order=4, int max_factors_common_order=4, bool use_symbolic=false)
{
	execute<D3Q27_SRT<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q27_MRT1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q27_MRT2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q27_CLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q27_CLBM2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q27_CuLBM1<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
	execute<D3Q27_CuLBM2<MM>>(pde_type,  order, max_factors_common_order, use_symbolic);
}


int main(int argc, char* argv[])
{
	int O=4,MFCO=O;
	bool use_symbolic=true;

	d1q3(ADE,O,MFCO,use_symbolic);

	d1q3(NSE,O,MFCO,use_symbolic);

	d2q5(ADE,O,MFCO,use_symbolic);

	d2q9(NSE,O,MFCO,use_symbolic);

	d3q7(ADE,O,MFCO,use_symbolic);

	d3q27(NSE,O,MFCO,use_symbolic);
}
